@@ -5,7 +5,7 @@ ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2019, Free Software Foundation, Inc.
+Copyright (C) 2016-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -191,7 +191,7 @@ struct argp_option program_options[] =
       UI_KEY_CHANGE,
       "STR",
       0,
-      "Change pixel values `from_1:to_1,from_2:to_2`.",
+      "Change pixel values 'from_1:to_1,from_2:to_2'.",
       UI_GROUP_FLUX,
       &p->changestr,
       GAL_TYPE_STRING,
