@@ -5,7 +5,7 @@ Convolve is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -77,7 +77,7 @@ complextoreal(double *c, size_t size, int action, double **output)
       break;
     default:
       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s so we can "
-            "correct it. The `action' code %d is not recognized", __func__,
+            "correct it. The 'action' code %d is not recognized", __func__,
             PACKAGE_BUGREPORT, action);
     }
 }
@@ -245,7 +245,7 @@ frequency_make_padded_complex(struct convolveparams *p)
     roundoff errors.
 
     NOTE: The padding to the input image (on the first axis for example)
-          was `p->kernel->dsize[0]-1'. Since `p->kernel->dsize[0]' is
+          was 'p->kernel->dsize[0]-1'. Since 'p->kernel->dsize[0]' is
           always odd, the padding will always be even.  */
 void
 removepaddingcorrectroundoff(struct convolveparams *p)
@@ -274,7 +274,7 @@ removepaddingcorrectroundoff(struct convolveparams *p)
       hi1 = ( p->kernel->dsize[1] - 1 )/2;
     }
 
-  /* To start with, `start' points to the first pixel in the final
+  /* To start with, 'start' points to the first pixel in the final
      image: */
   start=&rpad[hi0*ps1+hi1];
   for(i=0;i<isize[0];++i)
@@ -373,21 +373,21 @@ correctdeconvolve(struct convolveparams *p, double **spatial)
   errno=0;
   n=malloc(ps0*ps1*sizeof *n);
   if(n==NULL)
-    error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for `n'",
+    error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for 'n'",
           __func__, ps0*ps1*sizeof *n);
 
 
   /* Put the elements in their proper place: For example in one
-     dimention where the values are actually the true distances:
+     dimension where the values are actually the true distances:
 
         s[0]=0, s[1]=1, s[2]=2, s[3]=3, s[4]=4, s[5]=5
 
-     We want the value 0 to be in the `center'. Note that `s' is
+     We want the value 0 to be in the 'center'. Note that 's' is
      periodic, for example the next 6 elements have distances:
 
         s[6]=0, s[7]=1, s[8]=2, s[9]=3, s[10]=4, s[11]=5
 
-     So a `center'ed array would be like:
+     So a 'center'ed array would be like:
 
         s[0]=4, s[1]=5, s[2]=0, s[3]=1, s[4]=2, s[5]=3
 
@@ -531,7 +531,7 @@ twodimensionfft(struct convolveparams *p, struct fftonthreadparams *fp,
   else if(forward1backwardn1==-1) multiple=1;
   else
     error(EXIT_FAILURE, 0, "%s: a bug! The value of the variable "
-          "`forward1backwardn1' is %d not 1 or 2. Please contact us at %s "
+          "'forward1backwardn1' is %d not 1 or 2. Please contact us at %s "
           "so we can find the cause of the problem and fix it", __func__,
           forward1backwardn1, PACKAGE_BUGREPORT);
 
@@ -642,7 +642,8 @@ convolve_frequency(struct convolveparams *p)
          don't need the array that is initially made. */
       dsize[0]=p->ps0; dsize[1]=p->ps1;
       data=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 2, dsize, NULL, 0,
-                          p->cp.minmapsize, NULL, NULL, NULL);
+                          p->cp.minmapsize, p->cp.quietmmap,
+                          NULL, NULL, NULL);
       free(data->array);
 
       /* Save the padded input image. */
@@ -761,6 +762,7 @@ void
 convolve(struct convolveparams *p)
 {
   gal_data_t *out, *check;
+  int multidim=p->input->ndim>1;
   struct gal_options_common_params *cp=&p->cp;
 
 
@@ -768,10 +770,10 @@ convolve(struct convolveparams *p)
   if(p->domain==CONVOLVE_DOMAIN_SPATIAL)
     {
       /* Prepare the mesh structure. */
-      gal_tile_full_two_layers(p->input, &cp->tl);
+      if(multidim) gal_tile_full_two_layers(p->input, &cp->tl);
 
       /* Save the tile IDs if they are requested. */
-      if(cp->tl.tilecheckname)
+      if(multidim && cp->tl.tilecheckname)
         {
           check=gal_tile_block_check_tiles(cp->tl.tiles);
           gal_fits_img_write(check, cp->tl.tilecheckname, NULL, PROGRAM_NAME);
@@ -782,8 +784,10 @@ convolve(struct convolveparams *p)
          want to do spatial domain convolution with this Convolve program
          is edge correction. So by default we assume it and will only
          ignore it if the user asks.*/
-      out=gal_convolve_spatial(cp->tl.tiles, p->kernel, cp->numthreads,
-                               !p->noedgecorrection, cp->tl.workoverch);
+      out=gal_convolve_spatial(multidim ? cp->tl.tiles : p->input, p->kernel,
+                               cp->numthreads,
+                               multidim ? !p->noedgecorrection : 1,
+                               multidim ? cp->tl.workoverch : 1 );
 
       /* Clean up: free the actual input and replace it's pointer with the
          convolved dataset to save as output. */
@@ -795,6 +799,20 @@ convolve(struct convolveparams *p)
     convolve_frequency(p);
 
   /* Save the output (which is in p->input) array. */
-  gal_fits_img_write_to_type(p->input, cp->output, NULL, PROGRAM_NAME,
-                             cp->type);
+  if(p->input->ndim==1)
+    gal_table_write(p->input, NULL, p->cp.tableformat, p->cp.output,
+                    "CONVOLVED", 0);
+  else
+    gal_fits_img_write_to_type(p->input, cp->output, NULL, PROGRAM_NAME,
+                               cp->type);
+
+  /* Write Convolve's parameters as keywords into the first extension of
+     the output. */
+  if( gal_fits_name_is_fits(p->cp.output) )
+    {
+      gal_fits_key_write_filename("input", p->filename, &cp->okeys, 1);
+      gal_fits_key_write_config(&cp->okeys, "Convolve configuration",
+                                "CONVOLVE-CONFIG", cp->output, "0");
+    }
+  printf("  - Output: %s\n", p->cp.output);
 }
