@@ -5,7 +5,7 @@ CosmicCalculator is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2019, Free Software Foundation, Inc.
+Copyright (C) 2016-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -44,6 +44,20 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "obsline",
+      UI_KEY_OBSLINE,
+      "STR,FLT",
+      0,
+      "Redshift from line and observed wavelength.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->obsline,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_parse_obsline
+    },
     {
       "H0",
       UI_KEY_H0,
@@ -100,11 +114,45 @@ struct argp_option program_options[] =
 
 
 
+
+    /* Output options */
+    {
+      "listlines",
+      UI_KEY_LISTLINES,
+      0,
+      0,
+      "List known spectral lines.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->listlines,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
+
+
+
     {
       0, 0, 0, 0,
       "Specific calculations",
       UI_GROUP_SPECIFIC
     },
+    {
+      "usedredshift",
+      UI_KEY_USEDREDSHIFT,
+      0,
+      0,
+      "Used redshift in this run.",
+      UI_GROUP_SPECIFIC,
+      &p->specific,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value,
+    },
     {
       "agenow",
       UI_KEY_AGENOW,
@@ -273,6 +321,21 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       ui_add_to_single_value,
     },
+    {
+      "lineatz",
+      UI_KEY_LINEATZ,
+      "STR/FLT",
+      0,
+      "Wavelength of given line at chosen redshift",
+      UI_GROUP_SPECIFIC,
+      &p->specific,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      ui_add_to_single_value,
+    },
+
 
 
     {0}
