@@ -5,7 +5,7 @@ Fits is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -25,6 +25,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <errno.h>
 #include <error.h>
 #include <string.h>
+#include <unistd.h>
 
 #include <gnuastro/list.h>
 #include <gnuastro/fits.h>
@@ -54,11 +55,11 @@ fits_has_error(struct fitsparams *p, int actioncode, char *string, int status)
     case FITS_ACTION_UPDATE:        action="updated";      break;
     case FITS_ACTION_WRITE:         action="written";      break;
     case FITS_ACTION_COPY:          action="copied";       break;
-    case FITS_ACTION_REMOVE:        action="renoved";      break;
+    case FITS_ACTION_REMOVE:        action="removed";      break;
 
     default:
-      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at `%s' so we "
-            "can fix this problem. The value of `actioncode' must not be %d",
+      error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at '%s' so we "
+            "can fix this problem. The value of 'actioncode' must not be %d",
             __func__, PACKAGE_BUGREPORT, actioncode);
     }
 
@@ -85,6 +86,7 @@ fits_print_extension_info(struct fitsparams *p)
 {
   uint16_t *ui16;
   fitsfile *fptr;
+  int hasblankname=0;
   gal_data_t *cols=NULL, *tmp;
   char **tstra, **estra, **sstra;
   size_t i, numext, *dsize, ndim;
@@ -109,17 +111,17 @@ fits_print_extension_info(struct fitsparams *p)
   /* Allocate all the columns (in reverse order, since this is a simple
      linked list). */
   gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_STRING, 1, &numext, NULL, 1,
-                          p->cp.minmapsize, "HDU_SIZE", "name", "Size of "
-                          "image or table number of rows and columns.");
+                          -1, 1, "HDU_SIZE", "name", "Size of image or table "
+                          "number of rows and columns.");
   gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_STRING, 1, &numext, NULL, 1,
-                          p->cp.minmapsize, "HDU_TYPE", "name", "Image "
-                          "data type or `table' format (ASCII or binary).");
+                          -1, 1, "HDU_TYPE", "name", "Image data type or "
+                          "'table' format (ASCII or binary).");
   gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_STRING, 1, &numext, NULL, 1,
-                          p->cp.minmapsize, "EXTNAME", "name",
-                          "Extension name of this HDU (EXTNAME in FITS).");
+                          -1, 1, "EXTNAME", "name", "Extension name of this "
+                          "HDU (EXTNAME in FITS).");
   gal_list_data_add_alloc(&cols, NULL, GAL_TYPE_UINT16, 1, &numext, NULL, 1,
-                          p->cp.minmapsize, "HDU_INDEX", "count", "Index "
-                          "(starting from zero) of each HDU (extension).");
+                          -1, 1, "HDU_INDEX", "count", "Index (starting from "
+                          "zero) of each HDU (extension).");
 
 
   /* Keep pointers to the array of each column for easy writing. */
@@ -153,7 +155,7 @@ fits_print_extension_info(struct fitsparams *p)
           break;
 
         default:
-          error(EXIT_FAILURE, 0, "%s: a bug! the `hdutype' code %d not "
+          error(EXIT_FAILURE, 0, "%s: a bug! the 'hdutype' code %d not "
                 "recognized", __func__, hdutype);
         }
 
@@ -168,6 +170,7 @@ fits_print_extension_info(struct fitsparams *p)
 
         case KEY_NO_EXIST:
           sprintf(extname, "%s", GAL_BLANK_STRING);
+          hasblankname=1;
           status=0;
           break;
 
@@ -177,9 +180,9 @@ fits_print_extension_info(struct fitsparams *p)
       status=0;
 
 
-      /* Write the size into a string. `sprintf' returns the number of
-         written characters (excluding the `\0'). So for each dimension's
-         size that is written, we add to `nc' (the number of
+      /* Write the size into a string. 'sprintf' returns the number of
+         written characters (excluding the '\0'). So for each dimension's
+         size that is written, we add to 'nc' (the number of
          characters). Note that FITS allows blank extensions, in those
          cases, return "0". */
       if(ndim>0)
@@ -229,11 +232,14 @@ fits_print_extension_info(struct fitsparams *p)
   if(!p->cp.quiet)
     {
       printf("%s\nRun on %s-----\n", PROGRAM_STRING, ctime(&p->rawtime));
-      printf("HDU (extension) information: `%s'.\n", p->filename);
-      printf(" Column 1: Index (counting from 0, usable with `--hdu').\n");
-      printf(" Column 2: Name (`EXTNAME' in FITS standard, usable with "
-             "`--hdu').\n");
-      printf(" Column 3: Image data type or `table' format (ASCII or "
+      printf("HDU (extension) information: '%s'.\n", p->filename);
+      printf(" Column 1: Index (counting from 0, usable with '--hdu').\n");
+      printf(" Column 2: Name ('EXTNAME' in FITS standard, usable with "
+             "'--hdu').\n");
+      if(hasblankname)
+        printf("           ('%s' means that no name is specified for this "
+               "HDU)\n", GAL_BLANK_STRING);
+      printf(" Column 3: Image data type or 'table' format (ASCII or "
              "binary).\n");
       printf(" Column 4: Size of data in HDU.\n");
       printf("-----\n");
@@ -270,6 +276,31 @@ fits_hdu_number(struct fitsparams *p)
 
 
 
+static void
+fits_datasum(struct fitsparams *p)
+{
+  int status=0;
+  fitsfile *fptr;
+  unsigned long datasum, hdusum;
+
+  /* Read the first extension (necessary for reading the rest). */
+  fptr=gal_fits_hdu_open(p->filename, p->cp.hdu, READONLY);
+
+  /* Calculate the checksum and datasum of the opened HDU. */
+  fits_get_chksum(fptr, &datasum, &hdusum, &status);
+  gal_fits_io_error(status, "estimating datasum");
+
+  /* Close the file. */
+  fits_close_file(fptr, &status);
+
+  /* Print the datasum */
+  printf("%ld\n", datasum);
+}
+
+
+
+
+
 static void
 fits_hdu_remove(struct fitsparams *p, int *r)
 {
@@ -299,17 +330,45 @@ fits_hdu_remove(struct fitsparams *p, int *r)
 
 
 
+/* This is similar to the library's 'gal_fits_open_to_write', except that
+   it won't create an empty first extension.*/
+fitsfile *
+fits_open_to_write_no_blank(char *filename)
+{
+  int status=0;
+  fitsfile *fptr;
+
+  /* When the file exists just open it. Otherwise, create the file. But we
+     want to leave the first extension as a blank extension and put the
+     image in the next extension to be consistent between tables and
+     images. */
+  if(access(filename,F_OK) == -1 )
+    {
+      /* Create the file. */
+      if( fits_create_file(&fptr, filename, &status) )
+        gal_fits_io_error(status, NULL);
+    }
+
+  /* Open the file, ready for later steps. */
+  if( fits_open_file(&fptr, filename, READWRITE, &status) )
+    gal_fits_io_error(status, NULL);
+
+  /* Return the pointer. */
+  return fptr;
+}
+
+
+
+
+
 static void
 fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
 {
   char *hdu;
-  fitsfile *in, *out;
   int status=0, hdutype;
+  fitsfile *in, *out=NULL;
   gal_list_str_t *list = cut1_copy0 ? p->cut : p->copy;
 
-  /* Open the output file. */
-  out=gal_fits_open_to_write(p->cp.output);
-
   /* Copy all the given extensions. */
   while(list)
     {
@@ -320,12 +379,20 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
       in=gal_fits_hdu_open(p->filename, hdu,
                            cut1_copy0 ? READWRITE : READONLY);
 
+      /* If the output isn't opened yet, open it.  */
+      if(out==NULL)
+        out = ( ( gal_fits_hdu_format(p->filename, hdu)==IMAGE_HDU
+                  && p->primaryimghdu )
+                ? fits_open_to_write_no_blank(p->cp.output)
+                : gal_fits_open_to_write(p->cp.output) );
+
+
       /* Copy to the extension. */
       if( fits_copy_hdu(in, out, 0, &status) )
         *r=fits_has_error(p, FITS_ACTION_COPY, hdu, status);
       status=0;
 
-      /* If this is a `cut' operation, then remove the extension. */
+      /* If this is a 'cut' operation, then remove the extension. */
       if(cut1_copy0)
         {
           if( fits_delete_hdu(in, &hdutype, &status) )
@@ -338,7 +405,7 @@ fits_hdu_copy(struct fitsparams *p, int cut1_copy0, int *r)
     }
 
   /* Close the output file. */
-  fits_close_file(out, &status);
+  if(out) fits_close_file(out, &status);
 }
 
 
@@ -352,16 +419,19 @@ fits(struct fitsparams *p)
 
   switch(p->mode)
     {
-    /* Keywords, we have a separate set of functions in `keywords.c'. */
+    /* Keywords, we have a separate set of functions in 'keywords.c'. */
     case FITS_MODE_KEY:
       r=keywords(p);
       break;
 
     /* HDU, functions defined here. */
     case FITS_MODE_HDU:
+
       /* Options that must be called alone. */
       if(p->numhdus)
         fits_hdu_number(p);
+      else if(p->datasum)
+        fits_datasum(p);
 
       /* Options that can be called together. */
       else
