@@ -5,7 +5,7 @@ MakeCatalog is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -42,6 +42,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/permutation.h>
 
 #include <gnuastro-internal/timing.h>
+#include <gnuastro-internal/checkset.h>
 
 #include "main.h"
 #include "mkcatalog.h"
@@ -129,12 +130,12 @@ mkcatalog_single_object(void *in_prm)
     {
       /* Allocate the space to keep the upper-limit values. */
       pp.up_vals = gal_data_alloc(NULL, GAL_TYPE_FLOAT32, 1, &p->upnum,
-                                  NULL, 0, p->cp.minmapsize, NULL, NULL,
-                                  NULL);
+                                  NULL, 0, p->cp.minmapsize, p->cp.quietmmap,
+                                  NULL, NULL, NULL);
 
       /* Set the blank checked flag to 1. By definition, this dataset won't
-         have any blank values. Also `flag' is initialized to `0'. So we
-         just have to set the checked flag (`GAL_DATA_FLAG_BLANK_CH') to
+         have any blank values. Also 'flag' is initialized to '0'. So we
+         just have to set the checked flag ('GAL_DATA_FLAG_BLANK_CH') to
          one to inform later steps that there are no blank values. */
       pp.up_vals->flag |= GAL_DATA_FLAG_BLANK_CH;
     }
@@ -147,9 +148,12 @@ mkcatalog_single_object(void *in_prm)
     {
       /* For easy reading. Note that the object IDs start from one while
          the array positions start from 0. */
-      pp.ci     = NULL;
-      pp.object = tprm->indexs[i] + 1;
-      pp.tile   = &p->tiles[ tprm->indexs[i] ];
+      pp.ci       = NULL;
+      pp.object   = ( p->outlabs
+                      ? p->outlabs[tprm->indexs[i]]
+                      : tprm->indexs[i] + 1 );
+      pp.tile     = &p->tiles[   tprm->indexs[i] ];
+      pp.spectrum = &p->spectra[ tprm->indexs[i] ];
 
       /* Initialize the parameters for this object/tile. */
       parse_initialize(&pp);
@@ -175,9 +179,14 @@ mkcatalog_single_object(void *in_prm)
           parse_clumps(&pp);
         }
 
-      /* If the median is requested, another pass is necessary. */
-      if( p->oiflag[ OCOL_MEDIAN ] )
-        parse_median(&pp);
+      /* If an order-based calculation is requested, another pass is
+         necessary. */
+      if( p->oiflag[ OCOL_MEDIAN ]
+          || p->oiflag[ OCOL_SIGCLIPNUM ]
+          || p->oiflag[ OCOL_SIGCLIPSTD ]
+          || p->oiflag[ OCOL_SIGCLIPMEAN ]
+          || p->oiflag[ OCOL_SIGCLIPMEDIAN ])
+        parse_order_based(&pp);
 
       /* Calculate the upper limit magnitude (if necessary). */
       if(p->upperlimit) upperlimit_calculate(&pp);
@@ -266,19 +275,23 @@ mkcatalog_wcs_conversion(struct mkcatalogparams *p)
       /* Definitions */
       c=NULL;
 
-      /* Set `c' for the columns that must be corrected. Note that this
-         `switch' statement doesn't need any `default', because there are
+      /* Set 'c' for the columns that must be corrected. Note that this
+         'switch' statement doesn't need any 'default', because there are
          probably columns that don't need any correction. */
       switch(column->status)
         {
         case UI_KEY_W1:           c=p->wcs_vo;                break;
         case UI_KEY_W2:           c=p->wcs_vo->next;          break;
+        case UI_KEY_W3:           c=p->wcs_vo->next->next;    break;
         case UI_KEY_GEOW1:        c=p->wcs_go;                break;
         case UI_KEY_GEOW2:        c=p->wcs_go->next;          break;
+        case UI_KEY_GEOW3:        c=p->wcs_go->next->next;    break;
         case UI_KEY_CLUMPSW1:     c=p->wcs_vcc;               break;
         case UI_KEY_CLUMPSW2:     c=p->wcs_vcc->next;         break;
+        case UI_KEY_CLUMPSW3:     c=p->wcs_vcc->next->next;   break;
         case UI_KEY_CLUMPSGEOW1:  c=p->wcs_gcc;               break;
         case UI_KEY_CLUMPSGEOW2:  c=p->wcs_gcc->next;         break;
+        case UI_KEY_CLUMPSGEOW3:  c=p->wcs_gcc->next->next;   break;
         }
 
       /* Copy the elements into the output column. */
@@ -294,15 +307,17 @@ mkcatalog_wcs_conversion(struct mkcatalogparams *p)
       /* Definitions */
       c=NULL;
 
-      /* Set `c' for the columns that must be corrected. Note that this
-         `switch' statement doesn't need any `default', because there are
+      /* Set 'c' for the columns that must be corrected. Note that this
+         'switch' statement doesn't need any 'default', because there are
          probably columns that don't need any correction. */
       switch(column->status)
         {
         case UI_KEY_W1:           c=p->wcs_vc;                break;
         case UI_KEY_W2:           c=p->wcs_vc->next;          break;
+        case UI_KEY_W3:           c=p->wcs_vc->next->next;    break;
         case UI_KEY_GEOW1:        c=p->wcs_gc;                break;
         case UI_KEY_GEOW2:        c=p->wcs_gc->next;          break;
+        case UI_KEY_GEOW3:        c=p->wcs_gc->next->next;    break;
         }
 
       /* Copy the elements into the output column. */
@@ -427,7 +442,7 @@ mkcatalog_outputs_same_start(struct mkcatalogparams *p, int o0c1,
       gal_list_str_add(&comments, str, 0);
     }
 
-  /* Write the date. However, `ctime' is going to put a new-line character
+  /* Write the date. However, 'ctime' is going to put a new-line character
      in the end of its string, so we are going to remove it manually. */
   if( asprintf(&str, "%s started on %s", PROGRAM_NAME, ctime(&p->rawtime))<0 )
     error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
@@ -439,14 +454,15 @@ mkcatalog_outputs_same_start(struct mkcatalogparams *p, int o0c1,
   mkcatalog_write_inputs_in_comments(p, &comments, 1, 1);
 
 
-  /* Write other supplimentary information. */
+  /* Write other supplementary information. */
   if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
     {
-      if( asprintf(&str, "--------- Supplimentary information ---------")<0 )
+      if( asprintf(&str, "--------- Supplementary information ---------")<0 )
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_list_str_add(&comments, str, 0);
     }
 
+  /* Pixel area. */
   if(p->objects->wcs)
     {
       pixarea=gal_wcs_pixel_area_arcsec2(p->objects->wcs);
@@ -458,6 +474,7 @@ mkcatalog_outputs_same_start(struct mkcatalogparams *p, int o0c1,
         }
     }
 
+  /* Zeropoint magnitude */
   if(p->hasmag)
     {
       if( asprintf(&str, "Zeropoint magnitude: %.4f", p->zeropoint)<0 )
@@ -466,49 +483,53 @@ mkcatalog_outputs_same_start(struct mkcatalogparams *p, int o0c1,
     }
 
   /* Print surface brightness limits. */
-  if( !isnan(p->medstd) && !isnan(p->zeropoint) &&  !isnan(p->sfmagnsigma) )
+  if( !isnan(p->medstd) && !isnan(p->sfmagnsigma) )
     {
-      /* Per pixel. */
-      if( asprintf(&str, "%g sigma surface brightness (magnitude/pixel): "
-                   "%.3f", p->sfmagnsigma, ( -2.5f
-                                             *log10( p->sfmagnsigma
-                                                     * p->medstd )
-                                             + p->zeropoint ) )<0 )
-        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-      gal_list_str_add(&comments, str, 0);
-
-      /* Requested projected area: if a pixel area could be measured (a WCS
-         was given), then also estimate the surface brightness over one
-         arcsecond^2. From the pixel area, we know how many pixels are
-         necessary to fill the requested projected area (in
-         arcsecond^2). We also know that as the number of samples (pixels)
-         increases (to N), the noise increases by sqrt(N), see the full
-         discussion in the book. */
-      if(!isnan(pixarea) && !isnan(p->sfmagarea))
+      /* Only print magnitudes if a zeropoint is given. */
+      if( !isnan(p->zeropoint) )
         {
-          /* Prepare the comment/information. */
-          if(p->sfmagarea==1.0f)
-            tstr=NULL;
-          else
-            if( asprintf(&tstr, "%g-", p->sfmagarea)<0 )
-              error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-          if( asprintf(&str, "%g sigma surface brightness "
-                       "(magnitude/%sarcsec^2): %.3f", p->sfmagnsigma,
-                       tstr ? tstr : "",
-                       ( -2.5f * log10( p->sfmagnsigma
-                                        * p->medstd
-                                        * sqrt( p->sfmagarea / pixarea) )
-                         + p->zeropoint ) )<0 )
+          /* Per pixel. */
+          if( asprintf(&str, "%g sigma surface brightness (magnitude/pixel): "
+                       "%.3f", p->sfmagnsigma, ( -2.5f
+                                                 *log10( p->sfmagnsigma
+                                                         * p->medstd )
+                                                 + p->zeropoint ) )<0 )
             error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-
-          /* Add the final string/line to the catalog comments. */
           gal_list_str_add(&comments, str, 0);
 
-          /* Clean up (if necessary). */
-          if (tstr)
+          /* Requested projected area: if a pixel area could be measured (a
+             WCS was given), then also estimate the surface brightness over
+             one arcsecond^2. From the pixel area, we know how many pixels
+             are necessary to fill the requested projected area (in
+             arcsecond^2). We also know that as the number of samples
+             (pixels) increases (to N), the noise increases by sqrt(N), see
+             the full discussion in the book. */
+          if(!isnan(pixarea) && !isnan(p->sfmagarea))
             {
-              free(tstr);
-              tstr=NULL;
+              /* Prepare the comment/information. */
+              if(p->sfmagarea==1.0f)
+                tstr=NULL;
+              else
+                if( asprintf(&tstr, "%g-", p->sfmagarea)<0 )
+                  error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+              if( asprintf(&str, "%g sigma surface brightness "
+                           "(magnitude/%sarcsec^2): %.3f", p->sfmagnsigma,
+                           tstr ? tstr : "",
+                           ( -2.5f * log10( p->sfmagnsigma
+                                            * p->medstd
+                                            * sqrt( p->sfmagarea / pixarea) )
+                             + p->zeropoint ) )<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+
+              /* Add the final string/line to the catalog comments. */
+              gal_list_str_add(&comments, str, 0);
+
+              /* Clean up (if necessary). */
+              if (tstr)
+                {
+                  free(tstr);
+                  tstr=NULL;
+                }
             }
         }
 
@@ -519,7 +540,17 @@ mkcatalog_outputs_same_start(struct mkcatalogparams *p, int o0c1,
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_list_str_add(&comments, str, 0);
     }
+  else
+    {
+      gal_checkset_allocate_copy("No surface brightness calcuations "
+                                 "because no STD image used.", &str);
+      gal_list_str_add(&comments, str, 0);
+      gal_checkset_allocate_copy("Ask for column that uses the STD image, "
+                                 "or '--forcereadstd'.", &str);
+      gal_list_str_add(&comments, str, 0);
+    }
 
+  /* The count-per-second correction. */
   if(p->cpscorr>1.0f)
     {
       if( asprintf(&str, "Counts-per-second correction: %.3f", p->cpscorr)<0 )
@@ -527,11 +558,11 @@ mkcatalog_outputs_same_start(struct mkcatalogparams *p, int o0c1,
       gal_list_str_add(&comments, str, 0);
     }
 
+  /* Print upper-limit parameters. */
   if(p->upperlimit)
     upperlimit_write_comments(p, &comments, 1);
 
-
-
+  /* Start column metadata. */
   if(p->cp.tableformat==GAL_TABLE_FORMAT_TXT)
     {
       if( asprintf(&str, "--------- Table columns ---------")<0 )
@@ -561,7 +592,7 @@ sort_clumps_by_objid(struct mkcatalogparams *p)
   /* Make sure everything is fine. */
   if(p->hostobjid_c==NULL || p->numclumps_c==NULL)
     error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix the "
-          "problem. `p->hostobjid_c' and `p->numclumps_c' must not be "
+          "problem. 'p->hostobjid_c' and 'p->numclumps_c' must not be "
           "NULL.", __func__, PACKAGE_BUGREPORT);
 
 
@@ -606,37 +637,84 @@ sort_clumps_by_objid(struct mkcatalogparams *p)
 static void
 mkcatalog_write_outputs(struct mkcatalogparams *p)
 {
-  /*char *str;*/
+  size_t i, scounter;
+  char str[200], *fname;
   gal_list_str_t *comments;
+  int outisfits=gal_fits_name_is_fits(p->objectsout);
 
-
-  /* OBJECT catalog */
-  comments=mkcatalog_outputs_same_start(p, 0, "Detection");
-
-  /* Reverse the comments list (so it is printed in the same order here),
-     write the objects catalog and free the comments. */
-  gal_list_str_reverse(&comments);
-  gal_table_write(p->objectcols, comments, p->cp.tableformat,
-                  p->objectsout, "OBJECTS", 0);
-  gal_list_str_free(comments, 1);
-
-
-  /* CLUMPS catalog */
-  if(p->clumps)
+  /* If a catalog is to be generated. */
+  if(p->objectcols)
     {
-      /* Make the comments. */
-      comments=mkcatalog_outputs_same_start(p, 1, "Clumps");
+      /* OBJECT catalog */
+      comments=mkcatalog_outputs_same_start(p, 0, "Detection");
 
-      /* Reverse the comments and write the catalog. */
+      /* Reverse the comments list (so it is printed in the same order
+         here), write the objects catalog and free the comments. */
       gal_list_str_reverse(&comments);
-      gal_table_write(p->clumpcols, comments, p->cp.tableformat,
-                      p->clumpsout, "CLUMPS", 0);
+      gal_table_write(p->objectcols, comments, p->cp.tableformat,
+                      p->objectsout, "OBJECTS", 0);
       gal_list_str_free(comments, 1);
+
+
+      /* CLUMPS catalog */
+      if(p->clumps)
+        {
+          /* Make the comments. */
+          comments=mkcatalog_outputs_same_start(p, 1, "Clumps");
+
+          /* Write objects catalog
+             ---------------------
+
+             Reverse the comments list (so it is printed in the same order
+             here), write the objects catalog and free the comments. */
+          gal_list_str_reverse(&comments);
+          gal_table_write(p->clumpcols, comments, p->cp.tableformat,
+                          p->clumpsout, "CLUMPS", 0);
+          gal_list_str_free(comments, 1);
+        }
     }
 
+  /* Spectra. */
+  if(p->spectra)
+    {
+      /* Inform the user (Writing many FITS extensions can take
+         long). */
+      if(p->objectcols && outisfits)
+        printf("  - Catalog(s) complete, writing spectra.\n");
+
+      /* Start counting and writing the files. Note that due to some
+         conditions (for example in debugging), a 'p->spectra[i]' may not
+         actually contain any data. So we'll also count the number of
+         spectra that are written. */
+      scounter=0;
+      for(i=0;i<p->numobjects;++i)
+        if(p->spectra[i].ndim>0)
+          {
+            /* Increment the written spectra-counter. */
+            ++scounter;
+
+            /* Write the spectra based on the requested format. */
+            if(outisfits)
+              {
+                /* Write the table. */
+                sprintf(str, "SPECTRUM_%zu", i+1);
+                gal_table_write(&p->spectra[i], NULL, GAL_TABLE_FORMAT_BFITS,
+                                p->objectsout, str, 0);
+              }
+            else
+              {
+                sprintf(str, "-spec-%zu.txt", i+1);
+                fname=gal_checkset_automatic_output(&p->cp, p->objectsout,
+                                                    str);
+                gal_table_write(&p->spectra[i], NULL, GAL_TABLE_FORMAT_TXT,
+                                fname, NULL, 0);
+                free(fname);
+              }
+          }
+    }
 
   /* Configuration information. */
-  if(gal_fits_name_is_fits(p->objectsout))
+  if(outisfits)
     {
       gal_fits_key_write_filename("input", p->objectsfile, &p->cp.okeys, 1);
       gal_fits_key_write_config(&p->cp.okeys, "MakeCatalog configuration",
@@ -647,14 +725,34 @@ mkcatalog_write_outputs(struct mkcatalogparams *p)
   /* Inform the user */
   if(!p->cp.quiet)
     {
-      if(p->clumpsout && strcmp(p->clumpsout,p->objectsout))
+      if(p->objectcols)
         {
-          printf("  - Output objects catalog: %s\n", p->objectsout);
-          if(p->clumps)
-            printf("  - Output clumps catalog: %s\n", p->clumpsout);
+          if(p->clumpsout && strcmp(p->clumpsout,p->objectsout))
+            {
+              printf("  - Output objects catalog: %s\n", p->objectsout);
+              if(p->clumps)
+                printf("  - Output clumps catalog: %s\n", p->clumpsout);
+            }
+          else
+            printf("  - Catalog written to %s\n", p->objectsout);
+
+        }
+
+      if(p->spectra)
+        {
+          if(outisfits)
+            {
+              if(p->objectcols)
+                printf("  - Spectra in %zu extensions named 'SPECTRUM_NN'.\n",
+                       p->numobjects);
+              else
+                printf("  - Output: %s (Spectra in %zu extensions named "
+                       "'SPECTRUM_NN').\n)", p->objectsout, p->numobjects);
+            }
+          else
+            printf("  - Spectra in %zu files with '-spec-NN.txt' suffix.\n",
+                   p->numobjects);
         }
-      else
-        printf("  - Catalog written to %s\n", p->objectsout);
     }
 }
 
