@@ -3,9 +3,9 @@ MakeNoise - Add noise to a dataset.
 MakeNoise is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -20,13 +20,42 @@ General Public License for more details.
 You should have received a copy of the GNU General Public License
 along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 **********************************************************************/
-#ifndef IMCROPUI_H
-#define IMCROPUI_H
+#ifndef UI_H
+#define UI_H
+
+/* For common options groups. */
+#include <gnuastro-internal/options.h>
+
+
+
+
+
+/* Available letters for short options:
+
+   a c d f g j k l m n p r t u v w x y
+   A B C E G H J L O Q R W X Y
+*/
+enum option_keys_enum
+{
+  /* With short-option version. */
+  UI_KEY_SIGMA        = 's',
+  UI_KEY_INSTRUMENTAL = 'i',
+  UI_KEY_BACKGROUND   = 'b',
+  UI_KEY_ZEROPOINT    = 'z',
+  UI_KEY_ENVSEED      = 'e',
+
+  /* Only with long version (start with a value 1000, the rest will be set
+     automatically). */
+};
+
+
+
+
 
 void
-setparams(int argc, char *argv[], struct mknoiseparams *p);
+ui_read_check_inputs_setup(int argc, char *argv[], struct mknoiseparams *p);
 
 void
-freeandreport(struct mknoiseparams *p, struct timeval *t1);
+ui_free_report(struct mknoiseparams *p, struct timeval *t1);
 
 #endif
