@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2019, Free Software Foundation, Inc.
+Copyright (C) 2015-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -46,7 +46,7 @@ enum program_args_groups
 /* Available letters for short options:
 
    a b e f j p v w x z
-   B G J L R W X Y
+   B G J L W X Y
 */
 enum option_keys_enum
 {
@@ -70,6 +70,7 @@ enum option_keys_enum
   UI_KEY_INTERPOLATE  = 'i',
   UI_KEY_SKY          = 'y',
   UI_KEY_KERNEL       = 'k',
+  UI_KEY_CONTOUR      = 'R',
 
   /* Only with long version (start with a value 1000, the rest will be set
      automatically). */
@@ -100,6 +101,10 @@ enum option_keys_enum
   UI_KEY_CHECKSKY,
   UI_KEY_IGNOREBLANKINTILES,
   UI_KEY_SCLIPPARAMS,
+  UI_KEY_SIGCLIPNUMBER,
+  UI_KEY_SIGCLIPMEDIAN,
+  UI_KEY_SIGCLIPMEAN,
+  UI_KEY_SIGCLIPSTD,
 };
 
 
