@@ -1,5 +1,5 @@
 /* Hierarchical argument parsing, layered over getopt.
-   Copyright (C) 1995-2016 Free Software Foundation, Inc.
+   Copyright (C) 1995-2020 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
    Written by Miles Bader <miles@gnu.ai.mit.edu>.
 
@@ -14,7 +14,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */
 
 #ifndef _ARGP_H
 #define _ARGP_H
@@ -34,18 +34,6 @@
 # define __NTH(fct) fct __THROW
 #endif
 
-/* The __attribute__ feature is available in gcc versions 2.5 and later.
-   The __-protected variants of the attributes 'format' and 'printf' are
-   accepted by gcc versions 2.6.4 (effectively 2.7) and later.
-   We enable _GL_ATTRIBUTE_FORMAT only if these are supported too, because
-   gnulib and libintl do '#define printf __printf__' when they override
-   the 'printf' function.  */
-#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 7)
-# define _GL_ATTRIBUTE_FORMAT(spec) __attribute__ ((__format__ spec))
-#else
-# define _GL_ATTRIBUTE_FORMAT(spec) /* empty */
-#endif
-
 /* GCC 2.95 and later have "__restrict"; C99 compilers have
    "restrict", and "configure" may have defined "restrict".
    Other compilers use __restrict, __restrict__, and _Restrict, and
@@ -69,6 +57,9 @@ typedef int error_t;
 extern "C" {
 #endif
 
+/* Glibc documentation:
+   https://www.gnu.org/software/libc/manual/html_node/Argp.html */
+
 /* A description of a particular option.  A pointer to an array of
    these is passed in the OPTIONS field of an argp structure.  Each option
    entry can correspond to one long option and/or one short option; more
@@ -108,6 +99,37 @@ struct argp_option
      0), in which case, the previous entry + 1 is the default.  Automagic
      options such as --help are put into group -1.  */
   int group;
+
+  /* Pointer to value.
+     IMPORTANT: A Gnuastro addition. */
+  void *value;
+
+  /* Type of value (see lib/gnuastro/data.h).
+     IMPORTANT: A Gnuastro addition. */
+  int type;
+
+  /* Acceptable range of value (see lib/options.h).
+     IMPORTANT: A Gnuastro addition. */
+  int range;
+
+  /* Is this option mandatory? (see lib/options.h).
+     IMPORTANT: A Gnuastro addition. */
+  unsigned char mandatory;
+
+  /* Is value already set or not? (see lib/options.h).
+     IMPORTANT: A Gnuastro addition. */
+  unsigned char set;
+
+  /* Function to process the given value.
+     Arguments to the function:
+       struct argp_option *option: This argp_option.
+       char      *arg: String given by the user.
+       char *filename: Filename option was read from.
+       size_t  lineno: Line number of option in file.
+       void   *struct: Pointer to main program struct.
+
+     IMPORTANT: A Gnuastro addition. */
+  void *(*func)(struct argp_option *, char *, char *, size_t, void *);
 };
 
 /* The argument associated with this option is optional.  */
@@ -236,9 +258,9 @@ struct argp
      ARGP_KEY_ definitions below.  */
   argp_parser_t parser;
 
-  /* A string describing what other arguments are wanted by this program.  It
-     is only used by argp_usage to print the "Usage:" message.  If it
-     contains newlines, the strings separated by them are considered
+  /* If non-NULL, a string describing what other arguments are wanted by this
+     program.  It is only used by argp_usage to print the "Usage:" message.
+     If it contains newlines, the strings separated by them are considered
      alternative usage patterns, and printed on separate lines (lines after
      the first are prefix by "  or: " instead of "Usage:").  */
   const char *args_doc;
