@@ -4,7 +4,7 @@ Functions dealing with general aspects of all Gnuastro.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2016-2019, Free Software Foundation, Inc.
+Copyright (C) 2016-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -27,14 +27,14 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 
 /* The restrict keyword might not be present on some systems, so we are
-   redefining it here based on the checks in `configure.ac', see there for
+   redefining it here based on the checks in 'configure.ac', see there for
    more comments. */
 #define restrict __restrict
 
 
 /* Configuration macros: */
 
-#define GAL_CONFIG_VERSION                  "0.10"
+#define GAL_CONFIG_VERSION                  "0.12"
 
 #define GAL_CONFIG_HAVE_LIBGIT2             yes
 
