@@ -5,7 +5,8 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+     Sachin Kumar Singh <sachinkumarsingh092@gmail.com>
+Copyright (C) 2015-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -58,21 +59,35 @@ __BEGIN_C_DECLS  /* From C++ preparations */
 /**************     Function declarations     ******************/
 /***************************************************************/
 void
-gal_polygon_ordered_corners(double *in, size_t n, size_t *ordinds);
+gal_polygon_vertices_sort_convex(double *in, size_t n, size_t *ordinds);
+
+int
+gal_polygon_is_convex(double *v, size_t n);
 
 double
 gal_polygon_area(double *v, size_t n);
 
 int
-gal_polygon_pin(double *v, double *p, size_t n);
+gal_polygon_is_inside(double *v, double *p, size_t n);
+
+int
+gal_polygon_is_inside_convex(double *v, double *p, size_t n);
 
 int
 gal_polygon_ppropin(double *v, double *p, size_t n);
 
+int
+gal_polygon_is_counterclockwise(double *v, size_t n);
+
+int
+gal_polygon_to_counterclockwise(double *v, size_t n);
+
 void
 gal_polygon_clip(double *s, size_t n, double *c, size_t m,
                  double *o, size_t *numcrn);
 
+void
+gal_polygon_vertices_sort(double *in, size_t n, size_t *ordinds);
 
 __END_C_DECLS    /* From C++ preparations */
 
