@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2020, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -67,6 +67,8 @@ struct interpolate_params
   uint8_t                *thread_flags;
   int                        onlyblank;
   gal_list_void_t            *ngb_vals;
+  float (*metric)(size_t *, size_t *, size_t );
+
   struct gal_tile_two_layer_params *tl;
 };
 
@@ -87,11 +89,11 @@ interpolate_close_neighbors_on_thread(void *in_prm)
 
   /* Rest of variables. */
   void *nv;
-  float pdist;
+  float dist, pdist;
   uint8_t *b, *bf, *bb;
   gal_list_void_t *tvll;
   gal_list_dosizet_t *lQ, *sQ;
-  size_t ngb_counter, dist, pind, *dinc;
+  size_t ngb_counter, pind, *dinc;
   size_t i, index, fullind, chstart=0, ndim=input->ndim;
   gal_data_t *median, *tin, *tout, *tnear, *nearest=NULL;
   size_t size = (correct_index ? tl->tottilesinch : input->size);
@@ -118,7 +120,7 @@ interpolate_close_neighbors_on_thread(void *in_prm)
       nv=gal_pointer_increment(tvll->v, tprm->id*prm->numneighbors,
                                input->type);
       gal_list_data_add_alloc(&nearest, nv, tin->type, 1, &prm->numneighbors,
-                              NULL, 0, -1, NULL, NULL, NULL);
+                              NULL, 0, -1, 1, NULL, NULL, NULL);
       tin=tin->next;
     }
   gal_list_data_reverse(&nearest);
@@ -153,8 +155,8 @@ interpolate_close_neighbors_on_thread(void *in_prm)
          tiled dataset, the caller might want to interpolate the values of
          each channel separately (not mix values from different
          channels). In such a case, the tiles of each channel (and their
-         values in `input' are contiguous. So we need to correct
-         `tprm->indexs[i]' (which is the index over the whole tessellation,
+         values in 'input' are contiguous. So we need to correct
+         'tprm->indexs[i]' (which is the index over the whole tessellation,
          including all channels). */
       if(correct_index)
         {
@@ -233,7 +235,7 @@ interpolate_close_neighbors_on_thread(void *in_prm)
                  gal_dimension_index_to_coord(nind, ndim, dsize, ncoord);
 
                  /* Distance of this neighbor to the one to be filled. */
-                 dist=gal_dimension_dist_manhattan(icoord, ncoord, ndim);
+                 dist=prm->metric(icoord, ncoord, ndim);
 
                  /* Add this neighbor to the list. */
                  gal_list_dosizet_add(&lQ, &sQ, nind, dist);
@@ -246,7 +248,7 @@ interpolate_close_neighbors_on_thread(void *in_prm)
           /* If there are no more meshes to add to the queue, then this
              shows, there were not enough points for
              interpolation. Normally, this loop should only be exited
-             through the `currentnum>=numnearest' check above. */
+             through the 'currentnum>=numnearest' check above. */
           if(sQ==NULL)
             error(EXIT_FAILURE, 0, "%s: only %zu neighbors found while "
                   "you had asked to use %zu neighbors for close neighbor "
@@ -257,7 +259,9 @@ interpolate_close_neighbors_on_thread(void *in_prm)
       tout=prm->out;
       for(tnear=nearest; tnear!=NULL; tnear=tnear->next)
         {
-          /* Find the median and copy it. */
+          /* Find the median and copy it, but first, reset the flags (which
+             remain from the last time). */
+          tnear->flag &= ~(GAL_DATA_FLAG_SORT_CH | GAL_DATA_FLAG_BLANK_CH);
           median=gal_statistics_median(tnear, 1);
           memcpy(gal_pointer_increment(tout->array, fullind, tout->type),
                  median->array, gal_type_sizeof(tout->type));
@@ -320,14 +324,15 @@ interpolate_copy_input(gal_data_t *input, int aslinkedlist)
 
 
 
-/* Interpolate blank values in an array. If the `tl!=NULL', then it is
+/* Interpolate blank values in an array. If the 'tl!=NULL', then it is
    assumed that the tile values correspond to given tessellation. Such that
-   `input[i]' corresponds to `tiles[i]' in the tessellation. */
+   'input[i]' corresponds to 'tiles[i]' in the tessellation. */
 gal_data_t *
 gal_interpolate_close_neighbors(gal_data_t *input,
                                 struct gal_tile_two_layer_params *tl,
-                                size_t numneighbors, size_t numthreads,
-                                int onlyblank, int aslinkedlist)
+                                uint8_t metric, size_t numneighbors,
+                                size_t numthreads, int onlyblank,
+                                int aslinkedlist)
 {
   gal_data_t *tin, *tout;
   struct interpolate_params prm;
@@ -352,6 +357,21 @@ gal_interpolate_close_neighbors(gal_data_t *input,
   prm.num          = aslinkedlist ? gal_list_data_number(input) : 1;
 
 
+  /* Set the metric. */
+  switch(metric)
+    {
+    case GAL_INTERPOLATE_CLOSE_METRIC_RADIAL:
+      prm.metric=gal_dimension_dist_radial;
+      break;
+    case GAL_INTERPOLATE_CLOSE_METRIC_MANHATTAN:
+      prm.metric=gal_dimension_dist_manhattan;
+      break;
+    default:
+      error(EXIT_FAILURE, 0, "%s: %d is not a valid metric identifier",
+            __func__, metric);
+    }
+
+
   /* Flag the blank values. */
   prm.blanks=gal_blank_flag(input);
 
@@ -376,8 +396,8 @@ gal_interpolate_close_neighbors(gal_data_t *input,
 
   /* Allocate space for the (first) output. */
   prm.out=gal_data_alloc(NULL, input->type, input->ndim, input->dsize,
-                         input->wcs, 0, input->minmapsize, NULL,
-                         input->unit, NULL);
+                         input->wcs, 0, input->minmapsize,
+                         input->quietmmap, NULL, input->unit, NULL);
   gal_list_void_add(&prm.ngb_vals,
                     gal_pointer_allocate(input->type, ngbvnum, 0, __func__,
                                          "prm.ngb_vals"));
@@ -385,8 +405,8 @@ gal_interpolate_close_neighbors(gal_data_t *input,
 
   /* If we are given a list of datasets, make the necessary
      allocations. The reason we are doing this after a check of
-     `aslinkedlist' is that the `input' might have a `next' element, but
-     the caller might not have called `aslinkedlist'. */
+     'aslinkedlist' is that the 'input' might have a 'next' element, but
+     the caller might not have called 'aslinkedlist'. */
   prm.out->next=NULL;
   if(aslinkedlist)
     for(tin=input->next; tin!=NULL; tin=tin->next)
@@ -399,7 +419,7 @@ gal_interpolate_close_neighbors(gal_data_t *input,
         /* Allocate the output array for this node. */
         gal_list_data_add_alloc(&prm.out, NULL, tin->type, tin->ndim,
                                 tin->dsize, tin->wcs, 0, tin->minmapsize,
-                                NULL, tin->unit, NULL);
+                                tin->quietmmap, NULL, tin->unit, NULL);
 
         /* Allocate the space for the neighbor values of this input. */
         gal_list_void_add(&prm.ngb_vals,
@@ -526,17 +546,17 @@ gal_interpolate_1d_make_gsl_spline(gal_data_t *X, gal_data_t *Y, int type_1d)
     }
 
   /* Initializations. Note that if Y doesn't have any blank elements and is
-     already in `double' type, then we don't need to make a copy. */
+     already in 'double' type, then we don't need to make a copy. */
   Yd = ( (Yhasblank || Y->type!=GAL_TYPE_FLOAT64)
          ? gal_data_copy_to_new_type(Y, GAL_TYPE_FLOAT64)
          : Y );
   Xd = ( X
-         /* Has to be `Yhasblank', we KNOW X doesn't have blank values. */
+         /* Has to be 'Yhasblank', we KNOW X doesn't have blank values. */
          ? ( (Yhasblank || X->type!=GAL_TYPE_FLOAT64)
              ? gal_data_copy_to_new_type(X, GAL_TYPE_FLOAT64)
              : X )
          : gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, Y->dsize, NULL,
-                          0, -1, NULL, NULL, NULL) );
+                          0, -1, 1, NULL, NULL, NULL) );
 
   /* Fill in the X axis values while also removing NaN/blank elements. */
   c=0;
@@ -731,8 +751,8 @@ gal_interpolate_1d_blank(gal_data_t *in, int type_1d)
       }
       */
 
-      /* Set the blank flags, note that `GAL_DATA_FLAG_BLANK_CH' is already set
-         by the top call to `gal_blank_present'. */
+      /* Set the blank flags, note that 'GAL_DATA_FLAG_BLANK_CH' is already set
+         by the top call to 'gal_blank_present'. */
       if(hasblank)
         in->flag |=  GAL_DATA_FLAG_HASBLANK;
       else
